package com.zacharcher.juliatile {

/**
 *	@langversion ActionScript 3.0
 *	@playerversion Flash 10.0
 *
 *	@author Zach Archer
 *	@since  25.12.2008
 */
import mx.core.*;
import mx.containers.*;
import mx.controls.*;
import flash.display.*;
import flash.events.*;
import flash.filters.*;
import flash.net.*;
import flash.utils.*;

import com.zacharcher.juliatile.*;
import com.zacharcher.math.*;

public class Main extends MainView {
	
	//--------------------------------------
	// CLASS CONSTANTS
	//--------------------------------------
	
	[Embed(source="../../../../embed/shader/JuliaTile.pbj", mimeType="application/octet-stream")]
	        private static var JuliaTileShader:Class;		
	
	[Embed(source="../../../../embed/img/spaceneedle.jpg")]	// Picture of the Seattle Space Needle
	        private static var DefaultPic:Class;
	
	//--------------------------------------
	//  CONSTRUCTOR
	//--------------------------------------
	
	/**
	 *	@constructor
	 */
	public function Main()
	{
		super();
	}

	// Called from MainView.onCreationComplete
	public function init() :void {
		initFilter();
		initPic();
		initControls();
		initOscillators();
		initFileRef();
		
		addEventListener( Event.ENTER_FRAME, enterFrame );
	}
	
	private function initFilter() :void {
		_shader = new Shader( new JuliaTileShader() );
		_shader.precisionHint = ShaderPrecision.FAST;
		_filter = new ShaderFilter(_shader);
		_needsUpdate = true;	// apply the filter next frame
	}
	
	private function initPic() :void {
		var pic:BitmapAsset = new DefaultPic() as BitmapAsset;
		setImage( pic );
	}
	
	private function initControls() :void {
		for each( var slider:HSlider in allSliders ) {
			slider.addEventListener( Event.CHANGE, paramChange );
			updateFromSlider( slider );
		}
		loadNew.addEventListener( MouseEvent.CLICK, loadNewClick );
	}
	
	// Oscillators controls the animation
	private function initOscillators() :void {
		_oscs = new Dictionary();
		for each( var slider:HSlider in oscSliders ) {

			var min:Number = 0.0;
			var max:Number = 1.0;
			// Some sliders need less extreme values, so keep them on a short leash:
			switch( slider ) {
				case iterations: 	
					min = 0.1; max = 0.4; break;
				case biasX:
				case biasY:
					min = 0.25; max = 0.75; break;
			}
			
			var osc:Oscillator = new Oscillator( slider, min, max );
			_oscs[slider] = osc;
		}
	}
	
	private function initFileRef() :void {
		_file = new FileReference();
		// TODO: handle errors
		_file.addEventListener( Event.SELECT, fileSelect );
		_file.addEventListener( Event.COMPLETE, fileComplete );
	}
	
	//--------------------------------------
	//  PRIVATE VARIABLES
	//--------------------------------------
	private var _shader :Shader;
	private var _filter :ShaderFilter;
	private var _needsUpdate :Boolean = false;	// set this flag when _filter params change, need to re-apply the filter to see the changes
	
	private var _file :FileReference;
	private var _loader :Loader;	// loads bytes out of _file
	
	// oscillators
	private var _oscs :Dictionary;

	//--------------------------------------
	//  GETTER/SETTERS
	//--------------------------------------
	private function get isAnimating() :Boolean { return animate.selected; }
	private function get allSliders() :Array { return [complexX,complexY,biasX,biasY,iterations,fillType,rectWidth,rectHeight]; }
	// oscSliders are the ones that animate:
	private function get oscSliders() :Array { return [complexX,complexY,biasX,biasY,iterations]; }
	
	//--------------------------------------
	//  PUBLIC METHODS
	//--------------------------------------
	
	//--------------------------------------
	//  EVENT HANDLERS
	//--------------------------------------
	private function enterFrame( e:Event ) :void {
		// If a value has changed, then reapply the filter with its new settings.
		if( _needsUpdate ) {
			image.filters = [_filter];	// reapply to see the changes
			_needsUpdate = false;
		}
		
		if( isAnimating ) {
			for each( var slider:HSlider in oscSliders ) {
				_oscs[slider].tick();
				updateFromSlider( slider );
			}
		}
	}
	
	private function paramChange( e:Event ) :void {
		updateFromSlider( HSlider(e.currentTarget) );
	}
	
	//
	// FILE UPLOADS
	//
	private function loadNewClick( e:MouseEvent ) :void {
		// Show the "browse for a file" dialog box
		_file.browse( [new FileFilter("Images (*.jpg, *.jpeg, *.gif, *.png)", "*.jpg;*.jpeg;*.gif;*.png")] );		
	}
	private function fileSelect( e:Event ) :void {
		// The user has selected a file, so load it
		_file.load();
	}
	private function fileComplete( e:Event ) :void {
		// File bytes are loaded, now process these bytes with a Loader
		_loader = new Loader();
		_loader.contentLoaderInfo.addEventListener( Event.INIT, loaderInit, false, 0, true );
		_loader.loadBytes( _file.data );
	}
	private function loaderInit( e:Event ) :void {
		// Loader is ready (and knows its width & height) so we can display it now
		setImage( _loader );
	}
	
	//--------------------------------------
	//  PRIVATE & PROTECTED INSTANCE METHODS
	//--------------------------------------
	private function setImage( pic:Object ) :void {
		image.source = pic;
		// Leave a little margin so we can showcase the fill wraparound.
		rectWidth.value = pic.width * 0.667;
		updateFromSlider( rectWidth );
		rectHeight.value = pic.height * 0.667;
		updateFromSlider( rectHeight );
	}
	
	// When a slider updates (via user input or animation), we need to change the corresponding value in the filter.
	private function updateFromSlider( slider:HSlider ) :void {
		switch( slider ) {
			case complexX: 	_shader.data['complexX'].value = [complexX.value]; break;
			case complexY: 	_shader.data['complexY'].value = [complexY.value]; break;
			case biasX: 	_shader.data['biasX'].value = [biasX.value]; break;
			case biasY: 	_shader.data['biasY'].value = [biasY.value]; break;
			case iterations: _shader.data['iterations'].value = [iterations.value]; break;
			case rectWidth: _shader.data['rectWidth'].value = [rectWidth.value]; break;
			case rectHeight: _shader.data['rectHeight'].value = [rectHeight.value]; break;
			case fillType: 	_shader.data['fill'].value = [fillType.value]; break;
		}
		_needsUpdate = true;
	}
}

}

